------------------------------
--      Are you local?      --
------------------------------

local name = "Big Wigs Debugger"

local logging = nil
local logName = nil

local L = AceLibrary("AceLocale-2.2"):new("BigWigsDebugger")
local tablet = AceLibrary("Tablet-2.0")

----------------------------
--      Localization      --
----------------------------

L:RegisterTranslations("enUS", function() return {
	["Debug"] = true,
	["Click to toggle debugging."] = true,
	["|cff00ff00On|r"] = true,
	["|cffff0000Off|r"] = true,
	["|cff696969Idle|r"] = true,
	["|cffFF0000Log|r"] = true,
	["|cffFF0000Debug|r"] = true,
	["|cffeda55fClick|r to toggle logging. |cffeda55fCtrl+Click|r to toggle debugging."] = true,
	["Debugging set to %s."] = true,
	["BigWigs is not active."] = true,

	["Stuff that is bundled with Big Wigs that we can debug."] = true,
	["Builtin stuff"] = true,
	["Bars"] = true,
	["Toggle debugging when bars are started and stopped manually."] = true,
	["Messages"] = true,
	["Toggle debugging when messages are triggered."] = true,
	["Raid icon"] = true,
	["Toggle debugging when a raid icon is set by Big Wigs."] = true,
	["Target spotted"] = true,
	["Toggle debugging when a registered target is spotted."] = true,
	["Version query"] = true,
	["Toggle debugging when recieving version query syncs."] = true,
	["Module handling"] = true,
	["Debug events for BigWigs module handling."] = true,
	["EnableModule"] = true,
	["Toggle debugging of the EnableModule sync."] = true,
	["EnableExternal"] = true,
	["Toggle debugging of the EnableExternal sync."] = true,
	["RebootModule"] = true,
	["Toggle debugging of the RebootModule event."] = true,
	["All syncs"] = true,
	["Toggle debugging on all syncs."] = true,

	["Custom syncs"] = true,
	["Specify your own syncs to watch for."] = true,
	["Add sync"] = true,
	["Adds a sync to the list."] = true,
	["<sync>"] = true,
	["Remove sync"] = true,
	["Remove a sync from the list."] = true,
	["Remove %s."] = true,
	["Toggle debugging %s."] = true,
	["Logging"] = true,
	["Toggle logging the debug to saved variables."] = true,
	["Logging set to %s."] = true,
	["Clear logs"] = true,
	["Clear the saved variable logs."] = true,

	["Enabled modules"] = true,
	["Disabled modules"] = true,
	["Throttle"] = true,
	["Time"] = true,
	["Throttle timers"] = true,

	["Debug frame set to %s."] = true,
	["Debug frame"] = true,
	["Set debug chatframe."] = true,
	["<frameName>"] = true,

	["|cff00ff00%s|r added to the custom sync list."] = true,
	["|cff00ff00%s|r removed from the custom sync list."] = true,

	["Combat Events"] = true,
	["Toggle adding Combat Events to the debug and log."] = true,

} end)

----------------------------------
--      Addon Declaration       --
----------------------------------

BigWigsDebugger = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0", "AceEvent-2.0", "AceConsole-2.0")

BigWigsDebugger.debugFrame = ChatFrame5

-- FuBar stuff
BigWigsDebugger.version = "1.0." .. string.sub("$Revision: 35176 $", 12, -3)
BigWigsDebugger.date = string.sub("$Date: 2007-05-12 02:36:20 -0400 (Sat, 12 May 2007) $", 8, 17)
BigWigsDebugger.hasIcon = "Interface\\Icons\\Ability_Hunter_BeastCall"
BigWigsDebugger.canHideText = true

BigWigsDebugger:RegisterDB("BigWigsDebuggerDB")
BigWigsDebugger:RegisterDefaults("profile",{
	debugFrame = "ChatFrame5",
	bars = false,
	messages = false,
	raidicon = false,
	versionQuery = false,
	target = false,
	EnableModule = false,
	EnableExternal = false,
	RebootModule = false,
	customSyncList = {
		"BossEngaged",
	},
	customSyncs = {
		["BossEngaged"] = true,
	},
	combatEvents = true,
	logging = false,
	log = {},
})

local options = {
	type = "group",
	args = {
		debug = {
			type = "toggle",
			name = L["Debug"],
			desc = L["Click to toggle debugging."],
			get = function() return BigWigs:IsDebugging() end,
			set = function(v) BigWigsDebugger:ToggleDebugging(v) end,
			order = 101,
		},
		logging = {
			type = "toggle",
			name = L["Logging"],
			desc = L["Toggle logging the debug to saved variables."],
			get = function() return BigWigsDebugger.db.profile.logging end,
			set = function(v) BigWigsDebugger:ToggleLogging(v) end,
			order = 102,
		},
		clearlogs = {
			type = "execute",
			name = L["Clear logs"],
			desc = L["Clear the saved variable logs."],
			func = function() BigWigsDebugger:ClearLogs() end,
			order = 103,
		},
		debugframe = {
			type = "text",
			name = L["Debug frame"],
			desc = L["Set debug chatframe."],
			usage = L["<frameName>"],
			get = false,
			set = function(v) BigWigsDebugger:SetDebugFrame(v) end,
			order = 104,
		},
		spacer = { type = "header", order = 105 },
		modulehandling = {
			type = "group",
			name = L["Module handling"],
			desc = L["Debug events for BigWigs module handling."],
			order = 106,
			args = {
				enablemodule = {
					type = "toggle",
					name = L["EnableModule"],
					desc = L["Toggle debugging of the EnableModule sync."],
					get = function() return BigWigsDebugger.db.profile.EnableModule end,
					set = function(v) BigWigsDebugger.db.profile.EnableModule = v end
				},
				enableexternal = {
					type = "toggle",
					name = L["EnableExternal"],
					desc = L["Toggle debugging of the EnableExternal sync."],
					get = function() return BigWigsDebugger.db.profile.EnableExternal end,
					set = function(v) BigWigsDebugger.db.profile.EnableExternal = v end
				},
				rebootmodule = {
					type = "toggle",
					name = L["RebootModule"],
					desc = L["Toggle debugging of the RebootModule event."],
					get = function() return BigWigsDebugger.db.profile.RebootModule end,
					set = function(v) BigWigsDebugger:ToggleRebootModule(v) end
				},
			},
		},
		builtin = {
			type = "group",
			name = L["Builtin stuff"],
			desc = L["Stuff that is bundled with Big Wigs that we can debug."],
			order = 107,
			args = {
				allsync = {
					type = "toggle",
					name = L["All syncs"],
					desc = L["Toggle debugging on all syncs."],
					get = function() return BigWigsDebugger.db.profile.allSyncs end,
					set = function(v) BigWigsDebugger.db.profile.allSyncs = v end,
				},
				bars = {
					type = "toggle",
					name = L["Bars"],
					desc = L["Toggle debugging when bars are started and stopped manually."],
					get = function() return BigWigsDebugger.db.profile.bars end,
					set = function(v) BigWigsDebugger:ToggleBars(v) end,
					hidden = function() return not BigWigsBars end,
				},
				messages = {
					type = "toggle",
					name = L["Messages"],
					desc = L["Toggle debugging when messages are triggered."],
					get = function() return BigWigsDebugger.db.profile.messages end,
					set = function(v) BigWigsDebugger:ToggleMessages(v) end,
					hidden = function() return not BigWigsMessages end,
				},
				raidicon = {
					type = "toggle",
					name = L["Raid icon"],
					desc = L["Toggle debugging when a raid icon is set by Big Wigs."],
					get = function() return BigWigsDebugger.db.profile.raidicon end,
					set = function(v) BigWigsDebugger:ToggleRaidIcon(v) end,
					hidden = function() return not BigWigsRaidIcon end,
				},
				target = {
					type = "toggle",
					name = L["Target spotted"],
					desc = L["Toggle debugging when a registered target is spotted."],
					get = function() return BigWigsDebugger.db.profile.target end,
					set = function(v) BigWigsDebugger:ToggleTargetSeen(v) end,
					hidden = function() return not BigWigsTargetMonitor end,
				},
				version = {
					type = "toggle",
					name = L["Version query"],
					desc = L["Toggle debugging when recieving version query syncs."],
					get = function() return BigWigsDebugger.db.profile.versionQuery end,
					set = function(v) BigWigsDebugger.db.profile.versionQuery = v end,
					hidden = function() return not BigWigsVersionQuery end,
				},
			},
		},
		custom = {
			type = "group",
			name = L["Custom syncs"],
			desc = L["Specify your own syncs to watch for."],
			order = 108,
			args = {},
		},
		combat = {
			type = "toggle",
			name = L["Combat Events"],
			desc = L["Toggle adding Combat Events to the debug and log."],
			get = function() return BigWigsDebugger.db.profile.combatEvents end,
			set = function(v) BigWigsDebugger:ToggleCombatEvents(v) end,
			order = 109,
		},

	},
}

BigWigsDebugger.OnMenuRequest = options

----------------------------------
--      Initialization          --
----------------------------------

local originalState = nil
function BigWigsDebugger:OnEnable()

	self:ToggleLogging(false)

	originalState = BigWigs:IsDebugging()
	if originalState == nil then
		self:ToggleDebugging(false)
	else
		self:ToggleDebugging(originalState)
	end

	self:SetDebugFrame(self.db.profile.debugFrame)

	self:UpdateSyncList()
end

function BigWigsDebugger:OnDisable()
	if BigWigs then BigWigs:SetDebugging(originalState) end
end

function BigWigsDebugger:SafeUnregister( event )
	-- Crappy AceEvent.
	if self:IsEventRegistered(event) then
		self:UnregisterEvent(event)
	end
end

----------------------------------
--      Custom sync handling    --
----------------------------------

function BigWigsDebugger:AddSync( sync )
	if not sync then return end
	sync = string.gsub(sync, " ", "")

	for k, syncName in pairs(self.db.profile.customSyncList) do
		if sync == syncName then return end
	end
	table.insert(self.db.profile.customSyncList, sync)

	self:Print(string.format(L["|cff00ff00%s|r added to the custom sync list."], sync))

	self:UpdateSyncList()
end

function BigWigsDebugger:RemoveSync( sync )
	for k, syncName in pairs(self.db.profile.customSyncList) do
		if sync == syncName then
			table.remove(self.db.profile.customSyncList, k)
			self:Print(string.format(L["|cff00ff00%s|r removed from the custom sync list."], sync))
			break
		end
	end
	self:UpdateSyncList()
end

function BigWigsDebugger:UpdateSyncList()
	options.args.custom.args = {}
	options.args.custom.args["add"] = {
		type = "text",
		name = L["Add sync"],
		desc = L["Adds a sync to the list."],
		usage = L["<sync>"],
		order = 101,
		get = false,
		set = function(v) BigWigsDebugger:AddSync(v) end,
	}

	options.args.custom.args["remove"] = {
		type = "group",
		name = L["Remove sync"],
		desc = L["Remove a sync from the list."],
		order = 102,
		args = {},
	}

	options.args.custom.args["spacer2"] = { type = "header", order = 103 }

	for _, sync in pairs(self.db.profile.customSyncList) do
		options.args.custom.args["remove"].args[sync] = {
			type = "execute",
			name = sync,
			desc = string.format(L["Remove %s."], sync),
			func = function() self:RemoveSync(sync) end,
		}
	end

	local counter = 4
	for _, sync in pairs(self.db.profile.customSyncList) do
		options.args.custom.args[sync] = {
			type = "toggle",
			name = sync,
			desc = string.format(L["Toggle debugging %s."], sync),
			order = 100 + counter,
			get = function() return self.db.profile.customSyncs[sync] end,
			set = function(v)
				self.db.profile.customSyncs[sync] = v
			end,
		}
	end
end

----------------------------------
--      Logging                 --
----------------------------------

function BigWigsDebugger:Log(text)
	if not text or not logging or not currentLog then return end
	table.insert(currentLog, "["..date("%H:%M:%S").."] - "..text)
end

function BigWigsDebugger:ClearLogs()
	for k in pairs(self.db.profile.log) do
		self.db.profile.log[k] = nil
	end
	self.db.profile.log = nil
	self.db.profile.log = {}
end

-- Most of the logging stuff ninjaed from Transcriptor, thanks ky ^^
function BigWigsDebugger:ToggleLogging( override )
	local shouldLog
	if override ~= nil and override == true or override == false then
		shouldLog = override
	else
		shouldLog = not self.db.profile.logging or nil
	end

	logName = nil
	currentLog = nil
	logging = shouldLog
	self.db.profile.logging = shouldLog

	if shouldLog then
		logName = "*** ["..date("%H:%M:%S").."] - "..GetRealZoneText().." : "..GetSubZoneText() .. " ***"
		if not self.db.profile.log[logName] then self.db.profile.log[logName] = {} end
		currentLog = self.db.profile.log[logName]
		self:RegisterMyEvents()
	else
		self:UnregisterMyEvents()
	end


	self:UpdateDisplay()
--	self:Print(string.format(L["Logging set to %s."], currentLog == nil and L["|cffff0000Off|r"] or L["|cff00ff00On|r"]))
end

----------------------------------
--      Debug toggling          --
----------------------------------

function BigWigsDebugger:RegisterMyEvents()
	self:RegisterEvent("BigWigs_RecvSync")
	self:RegisterEvent("BigWigs_CoreEnabled")
	self:RegisterEvent("BigWigs_CoreDisabled")
	self:RegisterEvent("BigWigs_ModuleLoaded")
	self:RegisterEvent("BigWigs_ModulePackLoaded")
	self:RegisterEvent("BigWigs_ModuleRegistered")
	self:RegisterEvent("Ace2_AddonEnabled")
	self:RegisterEvent("Ace2_AddonDisabled")

	self:ToggleAll()
end

function BigWigsDebugger:UnregisterMyEvents()
	-- don't unregister if we're debugging or logging
	if self:IsRunning() then return end

	self:SafeUnregister("BigWigs_RecvSync")
	self:SafeUnregister("BigWigs_CoreEnabled")
	self:SafeUnregister("BigWigs_CoreDisabled")
	self:SafeUnregister("BigWigs_ModuleLoaded")
	self:SafeUnregister("BigWigs_ModulePackLoaded")
	self:SafeUnregister("BigWigs_ModuleRegistered")
	self:SafeUnregister("Ace2_AddonEnabled")
	self:SafeUnregister("Ace2_AddonDisabled")

	self:ToggleAll()
end

function BigWigsDebugger:ToggleAll()
	self:ToggleRebootModule(self.db.profile.RebootModule)
	self:ToggleBars(self.db.profile.bars)
	self:ToggleMessages(self.db.profile.messages)
	self:ToggleRaidIcon(self.db.profile.raidicon)
	self:ToggleTargetSeen(self.db.profile.target)
	self:ToggleCombatEvents(self.db.profile.combatEvents)
end

function BigWigsDebugger:ToggleDebugging( debug )
	BigWigs:SetDebugging( debug )
	self:OnTextUpdate()
	if debug and self:IsRunning() then
		self:RegisterMyEvents()
	else
		self:UnregisterMyEvents()
	end

	self:Log("Debugging toggled ["..tostring(debug).."]")

	self:UpdateDisplay()

--	self:Print(string.format(L["Debugging set to %s."], BigWigs:IsDebugging() and L["|cff00ff00On|r"] or L["|cffff0000Off|r"]))
end

function BigWigsDebugger:ToggleTargetSeen( state )
	self.db.profile.target = state
	if state and self:IsRunning() then
		self:RegisterEvent("BigWigs_TargetSeen") -- unitname, unitid
		self:RegisterEvent("BigWigs_RegisterForTargetting") -- zonename, enabletrigger
	else
		self:SafeUnregister("BigWigs_TargetSeen")
		self:SafeUnregister("BigWigs_RegisterForTargetting")
	end
end

function BigWigsDebugger:ToggleRaidIcon( state )
	self.db.profile.raidicon = state
	if state and self:IsRunning() then
		self:RegisterEvent("BigWigs_SetRaidIcon") -- player name
		self:RegisterEvent("BigWigs_RemoveRaidIcon")
	else
		self:SafeUnregister("BigWigs_SetRaidIcon")
		self:SafeUnregister("BigWigs_RemoveRaidIcon")
	end
end

function BigWigsDebugger:ToggleMessages( state )
	self.db.profile.messages = state
	if state and self:IsRunning() then
		self:RegisterEvent("BigWigs_Message") -- text, color, noraidsay, sound
	else
		self:SafeUnregister("BigWigs_Message")
	end
end

function BigWigsDebugger:ToggleBars( state )
	self.db.profile.bars = state
	if state and self:IsRunning() then
		self:RegisterEvent("BigWigs_StartBar") -- module, text, time, icon
		self:RegisterEvent("BigWigs_StopBar") -- module, text
	else
		self:SafeUnregister("BigWigs_StartBar")
		self:SafeUnregister("BigWigs_StopBar")
	end
end

function BigWigsDebugger:ToggleRebootModule( state )
	self.db.profile.RebootModule = state
	if state and self:IsRunning() then
		self:RegisterEvent("BigWigs_RebootModule") -- module
	else
		self:SafeUnregister("BigWigs_RebootModule")
	end
end

function BigWigsDebugger:ToggleCombatEvents( state )
	self.db.profile.combatEvents = state
	if state and self:IsRunning() then
		-- register combat events
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		self:RegisterEvent("PLAYER_REGEN_DISABLED")
		self:RegisterEvent("CHAT_MSG_MONSTER_SAY")
		self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE")
		self:RegisterEvent("CHAT_MSG_RAID_BOSS_EMOTE")
		self:RegisterEvent("CHAT_MSG_MONSTER_WHISPER")
		self:RegisterEvent("CHAT_MSG_MONSTER_YELL")
		self:RegisterEvent("CHAT_MSG_SPELL_CREATURE_VS_CREATURE_DAMAGE")
		self:RegisterEvent("CHAT_MSG_SPELL_CREATURE_VS_CREATURE_BUFF")
		self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE")
		self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS")
		self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE")
		self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE")
		self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_PARTY_DAMAGE")
		self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_OTHER")
		self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE")
		self:RegisterEvent("PLAYER_TARGET_CHANGED")
		self:RegisterEvent("CHAT_MSG_COMBAT_FRIENDLY_DEATH")
		self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
	else
		-- unregister combat events
		self:SafeUnregister("PLAYER_REGEN_ENABLED")
		self:SafeUnregister("PLAYER_REGEN_DISABLED")
		self:SafeUnregister("CHAT_MSG_MONSTER_SAY")
		self:SafeUnregister("CHAT_MSG_MONSTER_EMOTE")
		self:SafeUnregister("CHAT_MSG_RAID_BOSS_EMOTE")
		self:SafeUnregister("CHAT_MSG_MONSTER_WHISPER")
		self:SafeUnregister("CHAT_MSG_MONSTER_YELL")
		self:SafeUnregister("CHAT_MSG_SPELL_CREATURE_VS_CREATURE_DAMAGE")
		self:SafeUnregister("CHAT_MSG_SPELL_CREATURE_VS_CREATURE_BUFF")
		self:SafeUnregister("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE")
		self:SafeUnregister("CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS")
		self:SafeUnregister("CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE")
		self:SafeUnregister("CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE")
		self:SafeUnregister("CHAT_MSG_SPELL_PERIODIC_PARTY_DAMAGE")
		self:SafeUnregister("CHAT_MSG_SPELL_AURA_GONE_OTHER")
		self:SafeUnregister("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE")
		self:SafeUnregister("PLAYER_TARGET_CHANGED")
		self:SafeUnregister("CHAT_MSG_COMBAT_FRIENDLY_DEATH")
		self:SafeUnregister("CHAT_MSG_COMBAT_HOSTILE_DEATH")
	end
end

function BigWigsDebugger:IsRunning()
	if BigWigs:IsDebugging() or logging then return true end
	return nil
end

----------------------------------
--      Debug printing          --
----------------------------------

function BigWigsDebugger:SetDebugFrame( frameName )
	local frame = getglobal(frameName)
	if not frame or type(frame.GetObjectType) ~= "function" or frame:GetObjectType() ~= "ScrollingMessageFrame" then
		frameName = "ChatFrame5"
	end
--	self:Print(string.format(L["Debug frame set to %s."], frameName))
	self.db.profile.debugFrame = frameName
	self.debugFrame = frame
	BigWigs.debugFrame = frame
	for name, module in BigWigs:IterateModules() do
		if type(module.IsDebugging) == "function" then
			module.debugFrame = frame
		end
	end
end

function BigWigsDebugger:PrintDebug( message, sender )
	if BigWigs:IsDebugging() then
		self:CustomPrint(1, 1, 1, self.debugFrame, nil, " ", date("%H:%M:%S").." ["..sender.."]: "..message)
	end
	self:Log("("..sender.."): "..message..".")
end

----------------------------------
--      Big Wigs Events         --
----------------------------------

function BigWigsDebugger:Ace2_AddonEnabled(addon)
	if not addon.core or not addon.core == BigWigs then return end
	local name = addon.name
	if not name and type(addon.ToString) == "function" then name = addon:ToString() end
	if not name then name = tostring(addon) end
	name = tostring(name)
	if not BigWigs:HasModule(name) then return end
	self:PrintDebug("Enabled: ["..name.."].", "Module")

	if type(addon.SetDebugging) == "function" then
		addon:SetDebugging(true)
		addon.debugFrame = getglobal(self.db.profile.debugFrame)
	end

	self:UpdateDisplay()
end

function BigWigsDebugger:Ace2_AddonDisabled(addon)
	if not addon.core or not addon.core == BigWigs then return end
	local name = addon.name
	if not name and type(addon.ToString) == "function" then name = addon:ToString() end
	if not name then name = tostring(addon) end
	name = tostring(name)
	if not BigWigs:HasModule(name) then return end
	self:PrintDebug("Disabled: ["..name.."].", "Module")

	if type(addon.SetDebugging) == "function" then
		addon:SetDebugging(false)
	end

	self:UpdateDisplay()
end

function BigWigsDebugger:BigWigs_ModulePackLoaded(name, failed)
	if not failed then
		self:PrintDebug("Module pack ["..name.."] loaded.", "Module")
	else
		self:PrintDebug("Module pack ["..name.."] failed to load.", "Module")
	end
	self:UpdateDisplay()
end

function BigWigsDebugger:BigWigs_ModuleLoaded(name, module)
	self:PrintDebug("Loaded: ["..name.."].", "Module")
	self:UpdateDisplay()
end

function BigWigsDebugger:BigWigs_ModuleRegistered(name)
	self:PrintDebug("Registered: ["..name.."].", "Module")
end

function BigWigsDebugger:BigWigs_CoreEnabled()
	self:PrintDebug("BigWigs Enabled", "Global")
	self:UpdateDisplay()
end

function BigWigsDebugger:BigWigs_CoreDisabled()
	self:PrintDebug("BigWigs Disabled", "Global")
	self:UpdateDisplay()
end

function BigWigsDebugger:BigWigs_RegisterForTargetting(zone, mobs)
	local mobStr = ""
	if mobs and type(mobs) == "table" then
		for k, v in pairs(mobs) do
			if mobStr ~= "" then mobStr = mobStr .. ", " end
			mobStr = mobStr..v
		end
	elseif mobs and type(mobs) == "string" then
		mobStr = mobs
	else
		mobStr = "INVALID"
	end
	if type(zone) ~= "string" then zone = tostring(zone) end
	self:PrintDebug("RegisterTarget ["..zone..": "..mobStr.."].", "Global")
end

function BigWigsDebugger:BigWigs_TargetSeen( unitName, unitId )
	self:PrintDebug("Target ["..unitName.."] ("..unitId..") spotted.", "Global")
end

function BigWigsDebugger:BigWigs_SetRaidIcon( player )
	self:PrintDebug("Setting raid icon on ["..player.."].", "Global")
end

function BigWigsDebugger:BigWigs_RemoveRaidIcon()
	self:PrintDebug("Clearing raid icon.", "Global")
end

function BigWigsDebugger:BigWigs_Message( text, color, noraidsay, sound )
	self:PrintDebug("Message ["..text.."]", "Global")
end

function BigWigsDebugger:BigWigs_StartBar( module, text, time, icon )
	self:PrintDebug("Starting bar ["..text..":"..time.."].", module:ToString())
end

function BigWigsDebugger:BigWigs_StopBar( module, text )
	self:PrintDebug("Stopping bar ["..text.."].", module:ToString())
end

function BigWigsDebugger:BigWigs_RebootModule( module )
	self:PrintDebug("Reboot "..module:ToString(), "Global")
end

BigWigsDebugger.syncHandlers = {}
-- Version query: BWVQ <zone>, BWVR <rev>
BigWigsDebugger.syncHandlers["BWVQ"] = function( sync, rest, nick )
	if not BigWigsDebugger.db.profile.versionQuery then return end
	BigWigsDebugger:PrintDebug("Version query on zone ["..rest.."].", nick)
end
BigWigsDebugger.syncHandlers["BWVR"] = function( sync, rest, nick )
	if not BigWigsDebugger.db.profile.versionQuery then return end
	BigWigsDebugger:PrintDebug("Version response ["..rest.."].", nick)
end
-- EnableModule/EnableExternal <modulename>
BigWigsDebugger.syncHandlers["EnableModule"] = function( sync, rest, nick )
	if not BigWigsDebugger.db.profile.EnableModule then return end
	BigWigsDebugger:PrintDebug("Module ["..rest.."] enable sync.", nick)
end
BigWigsDebugger.syncHandlers["EnableExternal"] = function( sync, rest, nick )
	if not BigWigsDebugger.db.profile.EnableExternal then return end
	BigWigsDebugger:PrintDebug("External ["..rest.."] enable sync.", nick)
end

-- Custom syncs
BigWigsDebugger.syncHandlers["BossEngaged"] = function( sync, rest, nick )
	BigWigsDebugger:PrintDebug("Boss ["..rest.."] engaged.", nick)
end

function BigWigsDebugger:BigWigs_RecvSync( sync, rest, nick )
	if self.syncHandlers[sync] then
		self.syncHandlers[sync](sync, rest, nick)
	elseif self.db.profile.allSyncs or self.db.profile.customSyncs[sync] then
		self:PrintDebug(sync..": "..rest, nick)
	end
end

-- Combat Events

function BigWigsDebugger:PLAYER_REGEN_ENABLED()
	self:PrintDebug("Combat", "Player Regen Enabled")
end

function BigWigsDebugger:PLAYER_REGEN_DISABLED()
	self:PrintDebug("Combat", "Player Regen Disabled")
end

function BigWigsDebugger:CHAT_MSG_MONSTER_EMOTE( msg, monster )
		self:PrintDebug("Combat", "Monster Emote: ["..monster.."]: "..msg )
end

function BigWigsDebugger:CHAT_MSG_RAID_BOSS_EMOTE( ... )
	local msg = strjoin(":", ...)
	msg = "Raid Boss Emote ["..msg.."]"
	self:PrintDebug("Combat", msg )	
end

function BigWigsDebugger:CHAT_MSG_MONSTER_SAY( msg, monster, rest )
	if rest then
		self:PrintDebug("Combat", "Monster Say: ["..monster.."]: "..msg )
	else
		self:PrintDebug("Combat", "Monster Say: ["..monster.."]: "..msg.." ("..rest..")")
	end
end

function BigWigsDebugger:CHAT_MSG_MONSTER_WHISPER( msg, monster )
	self:PrintDebug("Combat", "Monster Whisper: ["..monster.."]: "..msg )
end

function BigWigsDebugger:CHAT_MSG_MONSTER_YELL( msg, monster )
	self:PrintDebug("Combat", "Monster Yell: ["..monster.."]: "..msg )
end

function BigWigsDebugger:CHAT_MSG_SPELL_CREATURE_VS_CREATURE_DAMAGE( msg )
	self:PrintDebug("Combat", "[CvCdmg]: "..msg )
end

function BigWigsDebugger:CHAT_MSG_SPELL_CREATURE_VS_CREATURE_BUFF( msg )
	self:PrintDebug("Combat", "[CvCbuff]: "..msg )
end

function BigWigsDebugger:CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE( msg )
	self:PrintDebug("Combat", "[perHostPlrDmg]: "..msg )
end
--
function BigWigsDebugger:CHAT_MSG_SPELL_PERIODIC_CREATURE_BUFFS( msg )
	self:PrintDebug("Combat", "[perCBuffs]: "..msg )
end

function BigWigsDebugger:CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE( msg )
	self:PrintDebug("Combat", "[perSelfDmg]: "..msg )
end

function BigWigsDebugger:CHAT_MSG_SPELL_PERIODIC_FRIENDLYPLAYER_DAMAGE( msg )
	self:PrintDebug("Combat", "[perFriendDmg]: "..msg )
end

function BigWigsDebugger:CHAT_MSG_SPELL_PERIODIC_PARTY_DAMAGE( msg )
	self:PrintDebug("Combat", "[perPartyDmg]: "..msg )
end

function BigWigsDebugger:CHAT_MSG_SPELL_AURA_GONE_OTHER( msg )
	self:PrintDebug("Combat", "[AuraGoneOther]: "..msg )
end

function BigWigsDebugger:CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE( msg )
	self:PrintDebug("Combat", "[perCDmg]: "..msg )
end

function BigWigsDebugger:PLAYER_TARGET_CHANGED( msg )
	if (not UnitInRaid("target")) and UnitExists("target") then
		local name = UnitName("target")
		local level = UnitLevel("target")
		if UnitIsPlusMob("target") then 
			level = ("+"..level) 
		end
		local reaction
		if UnitIsFriend("target", "player") then
			reaction = "Friendly" 
		else
			reaction = "Hostile" 
		end
		local classification = UnitClassification("target")
		local creaturetype = UnitCreatureType("target")
		local typeclass
		if classification == "normal" then
			typeclass = creaturetype
		else 
			typeclass = (classification.." "..creaturetype)
		end
		local msg = string.format("%s %s (%s) - %s", level, reaction, typeclass, name)
		self:PrintDebug("Combat", "Target Changed: "..msg)
	end
end

function BigWigsDebugger:CHAT_MSG_COMBAT_FRIENDLY_DEATH( msg )
	self:PrintDebug( "Combat", "Friendly Death: "..msg )
end

function BigWigsDebugger:CHAT_MSG_COMBAT_HOSTILE_DEATH( msg )
	self:PrintDebug( "Combat", "Hostile Death: "..msg )
end

----------------------------------
--      FuBar Methods           --
----------------------------------

function BigWigsDebugger:OnTooltipUpdate()
	local cat = tablet:AddCategory(
		"columns", 2,
		"text", L["Debug"],
		"text2", L["Logging"]
	)
	cat:AddLine(
		"text", BigWigs:IsDebugging() and L["|cff00ff00On|r"] or L["|cffff0000Off|r"],
		"text2", currentLog == nil and L["|cffff0000Off|r"] or L["|cff00ff00On|r"]
	)

	local modules = ""
	local disabledModules = ""
	for name, module in BigWigs:IterateModules() do
		if BigWigs:IsModuleActive(module) and not module:IsBossModule() then
			if modules ~= "" then modules = modules .. ", " end
			modules = modules .. name
		elseif not module:IsBossModule() then
			if disabledModules ~= "" then disabledModules = disabledModules .. ", " end
			disabledModules = disabledModules .. name
		end
	end
	if modules ~= "" then
		local cat2 = tablet:AddCategory("text", L["Enabled modules"])
		cat2:AddLine("text", modules, "wrap", true)
	end
	if disabledModules ~= "" then
		local cat3 = tablet:AddCategory("text", L["Disabled modules"])
		cat3:AddLine("text", disabledModules, "wrap", true)
	end

	if BigWigsComm and type(BigWigsComm) == "table" and type(BigWigsComm.GetThrottleTable) == "function" then
		local throttles = BigWigsComm:GetThrottleTable()
		local throttleCat = tablet:AddCategory(
			"columns", 2,
			"text", L["Throttle"],
			"text2", L["Time"],
			"showWithoutChildren", false
		)
		for k, v in pairs(throttles) do
			throttleCat:AddLine("text", tostring(k), "text2", tostring(v))
		end
	end

	tablet:SetHint(L["|cffeda55fClick|r to toggle logging. |cffeda55fCtrl+Click|r to toggle debugging."])
end

function BigWigsDebugger:OnClick()
	if IsControlKeyDown() then
		self:ToggleDebugging(not BigWigs:IsDebugging())
	else
		self:ToggleLogging()
	end
end

function BigWigsDebugger:OnTextUpdate()
	local text = nil
	if BigWigs:IsDebugging() then
		text = L["|cffFF0000Debug|r"]
	end
	if currentLog then
		if text then 
			text = text .."&" .. L["|cffFF0000Log|r"]
		else 
			text = L["|cffFF0000Log|r"]
		end
	end
	if not text then
		text = L["|cff696969Idle|r"]
		self:SetIcon("Interface\\AddOns\\BigWigs_Debugger\\icon_off.tga")
	else
		self:SetIcon("Interface\\AddOns\\BigWigs_Debugger\\icon_on.tga")
	end
	self:SetText(text)
end

